import 'dart:convert';

import 'package:collection/collection.dart';

class Attribute {
  final String id;
  final String type;
  final int? size;
  final bool isRequired;
  final bool? signed;
  final bool? array;
  final List<String> filters;
  final dynamic defaultValue;
  Attribute({
    required this.id,
    required this.type,
    required this.isRequired,
    this.size,
    this.signed,
    this.array,
    this.filters = const [],
    this.defaultValue,
  });

  Attribute copyWith({
    String? id,
    String? type,
    int? size,
    bool? isRequired,
    bool? signed,
    bool? array,
    List<String>? filters,
    dynamic defaultValue,
  }) {
    return Attribute(
      id: id ?? this.id,
      type: type ?? this.type,
      size: size ?? this.size,
      isRequired: isRequired ?? this.isRequired,
      signed: signed ?? this.signed,
      array: array ?? this.array,
      filters: filters ?? this.filters,
      defaultValue: defaultValue ?? this.defaultValue,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'type': type,
      'size': size,
      'required': isRequired,
      'signed': signed,
      'array': array,
      'filters': filters,
      'default': defaultValue,
    };
  }

  factory Attribute.fromMap(Map<String, dynamic> map) {
    return Attribute(
      id: map['id'] ?? map['\$id'] ?? '',
      type: map['type'] ?? '',
      size: map['size']?.toInt() ?? 0,
      isRequired: map['required'] ?? false,
      signed: map['signed'] ?? false,
      array: map['array'] ?? false,
      filters: List<String>.from(map['filters']),
      defaultValue: map['default'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Attribute.fromJson(String source) =>
      Attribute.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Attribute(id: $id, type: $type, size: $size, required: $isRequired, signed: $signed, array: $array, filters: $filters, default: $defaultValue)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is Attribute &&
        other.id == id &&
        other.type == type &&
        other.size == size &&
        other.isRequired == isRequired &&
        other.signed == signed &&
        other.array == array &&
        other.defaultValue == defaultValue &&
        listEquals(other.filters, filters);
  }

  @override
  int get hashCode {
    return id.hashCode ^
        type.hashCode ^
        size.hashCode ^
        isRequired.hashCode ^
        signed.hashCode ^
        array.hashCode ^
        filters.hashCode;
  }
}
