import 'package:utopia_database/utopia_database.dart';
import 'package:test/test.dart';
import 'package:utopia_database_adapter_mariadb/utopia_database_adapter_mariadb.dart';

void main() {
  group('test Database', () {
    late Database database;
    MariaDB? mariadb;
    setUp(() async {
      // Additional setup goes here.
      if (mariadb != null) {
        return;
      }
      mariadb = await MariaDB.init(
          host: 'mariadb', port: 3306, user: 'root', password: 'root');
      database = Database(mariadb!);
      database.setNamespace('utopia_dart');
      database.setDefaultDatabase('mypersonaldatabase');
      return;
    });

    test('test create', () async {
      final created = await database.create();
      expect(created, true);
    });

    test('test exists', () async {
      final exists =
          await database.exists(database: database.getDefaultDatabase());
      expect(exists, true);
    });

    test('test create collection', () async {
      final collection = await database.createCollection('users', [
        Attribute(
            id: 'username',
            type: Database.varString,
            size: 255,
            isRequired: true),
      ], []);
      expect(collection!.id, 'users');
    });

    test('test create document', () async {
      final document = await database.createDocument(
          'users',
          Document(
              {"\$id": 'myspecialdocument', 'username': 'myspecialusername'}));
      expect(document.id, 'myspecialdocument');
    });

    test('test update document', () async {
      await database.updateDocument(
          'users',
          'myspecialdocument',
          Document({
            '\$id': 'myspecialdocument',
            'username': 'myspecialusernameupdated'
          }));

      final got = await database.getDocument('users', 'myspecialdocument');
      expect(got?.id, 'myspecialdocument');
      expect(got?.getAttribute('username', ''), 'myspecialusernameupdated');
    });

    test('delete document', () async {
      final deleted =
          await database.deleteDocument('users', 'myspecialdocument');
      expect(deleted, true);
    });

    test('test Delete Collection', () async {
      final deleted = await database.deleteCollection('users');
      expect(deleted, true);
    });
    test('test Delete', () async {
      final deleted = await database.delete(database.getDefaultDatabase());
      expect(deleted, true);
    });
  });
}
